/*
 *  Amanith Python wrapper
 *  Copyright (C) 2005-2007 by Sami Kystil <sami.kyostila@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


/*
 *  Add support for overloaded operators
 */

%warnfilter(389) Amanith::GVectBase::operator[];
%warnfilter(389) Amanith::GMatrix::operator[];
%warnfilter(389) Amanith::GQuat::operator[];

%define VECTOR_OPERATORS(ELEMENTS)
%extend Amanith::GVect<GReal, ELEMENTS>
{
  GReal __getitem__(int i) const
  {
    if ((i < 0) || (i > (ELEMENTS - 1)))
    {
      SWIG_Error(SWIG_IndexError, "Index out of range");
      return 0;
    }
    return self->Data()[i];
  }
  void __setitem__(int i, GReal value)
  {
    if ((i < 0) || (i > (ELEMENTS - 1)))
    {
      SWIG_Error(SWIG_IndexError, "Index out of range");
      return;
    }
    (*self)[i] = value;
  }
};
%enddef

VECTOR_OPERATORS(2);
VECTOR_OPERATORS(3);
VECTOR_OPERATORS(4);

/*
 * Quaternions
 */
%extend Amanith::GQuat<GReal>
{
  GReal __getitem__(int i) const
  {
    if ((i < 0) || (i > 4))
    {
      SWIG_Error(SWIG_IndexError, "Index out of range");
      return 0;
    }
    return self->Data()[i];
  }
  void __setitem__(int i, GReal value)
  {
    if ((i < 0) || (i > 4))
    {
      SWIG_Error(SWIG_IndexError, "Index out of range");
      return;
    }
    (*self)[i] = value;
  }
};

/*
 *  With matrices the column and row indices are given via vectors.
 */
%include "std_vector.i"

namespace std
{
  %template(CoordVector)        vector<int >;
}

%define MATRIX_OPERATORS(ROWS, COLS)
%extend Amanith::GMatrix<GReal, ROWS, COLS>
{
  GReal __getitem__(const std::vector<int>& c) const
  {
    return (*self)[c[0]][c[1]];
  }
  void __setitem__(const std::vector<int>& c, GReal value)
  {
    (*self)[c[0]][c[1]] = value;
  }
};
%enddef

MATRIX_OPERATORS(2, 2);
MATRIX_OPERATORS(2, 3);
MATRIX_OPERATORS(3, 3);
MATRIX_OPERATORS(3, 4);
MATRIX_OPERATORS(4, 3);
MATRIX_OPERATORS(4, 4);
