/*
 *  Amanith Python wrapper
 *  Copyright (C) 2005-2007 by Sami Kystil <sami.kyostila@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


/* Convenience typemaps */
#ifdef SWIGPYTHON

/* Make the common point types representable by tuples */
%typemap(in) const Amanith::GPoint2& (Amanith::GPoint2 temp)
{
  temp = Amanith::GPoint2(PyFloat_AsDouble(PyTuple_GetItem($input, 0)),
                          PyFloat_AsDouble(PyTuple_GetItem($input, 1)));
  $1 = &temp;
}

%typemap(in) const Amanith::GPoint3& (Amanith::GPoint3 temp)
{
  temp = Amanith::GPoint3(PyFloat_AsDouble(PyTuple_GetItem($input, 0)),
                          PyFloat_AsDouble(PyTuple_GetItem($input, 1)),
                          PyFloat_AsDouble(PyTuple_GetItem($input, 2)));
  $1 = &temp;
}

%typemap(in) const Amanith::GPoint4& (Amanith::GPoint4 temp)
{
  temp = Amanith::GPoint4(PyFloat_AsDouble(PyTuple_GetItem($input, 0)),
                          PyFloat_AsDouble(PyTuple_GetItem($input, 1)),
                          PyFloat_AsDouble(PyTuple_GetItem($input, 2)),
                          PyFloat_AsDouble(PyTuple_GetItem($input, 3)));
  $1 = &temp;
}

%typecheck(SWIG_TYPECHECK_POINTER) const Amanith::GPoint2&
{
  $1 = (PyTuple_Check($input) && PyTuple_Size($input) == 2) ? 1 : 0;
}

%typecheck(SWIG_TYPECHECK_POINTER) const Amanith::GPoint3&
{
  $1 = (PyTuple_Check($input) && PyTuple_Size($input) == 3) ? 1 : 0;
}

%typecheck(SWIG_TYPECHECK_POINTER) const Amanith::GPoint4&
{
  $1 = (PyTuple_Check($input) && PyTuple_Size($input) == 4) ? 1 : 0;
}

/* Same for vector types */
%typemap(in) const Amanith::GVector2& (Amanith::GVector2 temp)
{
  temp = Amanith::GVector2(PyFloat_AsDouble(PyTuple_GetItem($input, 0)),
                           PyFloat_AsDouble(PyTuple_GetItem($input, 1)));
  $1 = &temp;
}

%typemap(in) const Amanith::GVector3& (Amanith::GVector3 temp)
{
  temp = Amanith::GVector3(PyFloat_AsDouble(PyTuple_GetItem($input, 0)),
                           PyFloat_AsDouble(PyTuple_GetItem($input, 1)),
                           PyFloat_AsDouble(PyTuple_GetItem($input, 2)));
  $1 = &temp;
}

%typemap(in) const Amanith::GVector4& (Amanith::GVector4 temp)
{
  temp = Amanith::GVector4(PyFloat_AsDouble(PyTuple_GetItem($input, 0)),
                           PyFloat_AsDouble(PyTuple_GetItem($input, 1)),
                           PyFloat_AsDouble(PyTuple_GetItem($input, 2)),
                           PyFloat_AsDouble(PyTuple_GetItem($input, 3)));
  $1 = &temp;
}

%typecheck(SWIG_TYPECHECK_POINTER) const Amanith::GVector2&
{
  $1 = (PyTuple_Check($input) && PyTuple_Size($input) == 2) ? 1 : 0;
}

%typecheck(SWIG_TYPECHECK_POINTER) const Amanith::GVector3&
{
  $1 = (PyTuple_Check($input) && PyTuple_Size($input) == 3) ? 1 : 0;
}

%typecheck(SWIG_TYPECHECK_POINTER) const Amanith::GVector4&
{
  $1 = (PyTuple_Check($input) && PyTuple_Size($input) == 4) ? 1 : 0;
}

%typemap(in) Amanith::GReal& (Amanith::GReal temp)
{
  temp = PyFloat_AsDouble($input);
  $1 = &temp;
}

%typemap(out) Amanith::GReal&
{
  $result = PyFloat_FromDouble($1);
}

%apply Amanith::GReal& OUTPUT { Amanith::GReal& }
%apply Amanith::GPixelMap& INOUT { Amanith::GPixelMap& }

#endif /* SWIGPYTHON */

/*
 * Things that come in vectors
 */
%include "std_vector.i"

namespace std
{
  %template(GPoint2Vector)        vector<Amanith::GPoint2 >;
  %template(GPoint3Vector)        vector<Amanith::GPoint3 >;
  %template(GPoint4Vector)        vector<Amanith::GPoint4 >;
  %template(GKeyValueVector)      vector<Amanith::GKeyValue >;
  %template(GImpExpFeatureVector) vector<Amanith::GImpExpFeature >;
  %template(GPropertyVector)      vector<Amanith::GProperty >;
  %template(GProxyStateVector)    vector<Amanith::GProxyState >;
  %template(GRealVector)          vector<GReal >;
  %template(GHermiteKey1DVector)  vector<Amanith::GHermiteKey1D >;
}
